import unohelper, uno
import threading
from com.sun.star.awt import XActionListener
from com.sun.star.script.provider import XScriptProvider
import json
from urllib.request import urlopen
import traceback


"""
An event listener for the search button.
"""
class SearchClickedListener( unohelper.Base, XActionListener ):
    def __init__(self, control_container, thumbs_to_show):
        self.control = control_container
        self.thumbs = thumbs_to_show

    def actionPerformed(self, actionEvent):
        try:
            queryThread = OpenClipartQueryThread(self.control, 10, "load")
            queryThread.start()
        except Exception as e:
            showMessage("Error", "ERROR: \n" + str(e))

"""
An event listener for the next button.
"""
class ForwardClickedListener( unohelper.Base, XActionListener ):
    def __init__(self, control_container, thumbs_to_show):
        self.control = control_container
        self.thumbs = thumbs_to_show

    def actionPerformed(self, actionEvent):
        try:
            queryThread = OpenClipartQueryThread(self.control, 10, "next")
            queryThread.start()
        except Exception as e:
            showMessage("Error", "ERROR: \n" + str(e))

"""
An event listener for the previous button.
"""
class BackwardClickedListener( unohelper.Base, XActionListener ):
    def __init__(self, control_container, thumbs_to_show):
        self.control = control_container
        self.thumbs = thumbs_to_show

    def actionPerformed(self, actionEvent):
        try:
            queryThread = OpenClipartQueryThread(self.control, 10, "previous")
            queryThread.start()
        except Exception as e:
            showMessage("Error", "ERROR: \n" + str(e))

"""
An event listener for the thumbnail images.
"""
class ThumbnailClickedListener( unohelper.Base, XActionListener ):
    def __init__(self, control_container, thumbs_to_show, index):
        self.control = control_container
        self.thumbs = thumbs_to_show
        self.index = index

    def actionPerformed(self, actionEvent):
        try:
            sm = uno.getComponentContext().ServiceManager
            mspf = sm.createInstanceWithContext("com.sun.star.script.provider.MasterScriptProviderFactory", uno.getComponentContext())
            scriptPro = mspf.createScriptProvider("")
            xScript = scriptPro.getScript("vnd.sun.star.script:OCAL.basic.OpenClipartDialog.image.InsertImage?language=Basic&location=application")
            curData = MetadataContainer.getMetadata(self.index - 1)
            xScript.invoke((curData['url'], curData['title'], curData['description'], curData['detail_link'], curData['title']),(),())
            """
            sp = sm.createInstance("com.sun.star.script.provider.ScriptProvider")
            script = sp.getScript("vnd.sun.star.script:Standard.Module1.InsertImage?language=Basic&location=application")
            """
        except Exception as e:
            showMessage("Error", "ERROR: \n" + str(e))
            
"""
A threaded class in which we will do our lookups. It is done this way for better
user experience.
"""
class OpenClipartQueryThread(threading.Thread):
    """
    The constructor. All our parameters are set her.
    """
    def __init__(self, control, thumbs_to_show, page):
        threading.Thread.__init__(self)
        self.thumbs = thumbs_to_show
        self.page = page
        self.control = control

    """
    This is where the action takes place.
    """
    def run(self):
        try:
            i = 1
            while (i <= self.thumbs):
                self.control.getControl("image" + str(i)).getModel().ImageURL = "file:///home/dennis/.config/libreoffice/4/user/Scripts/python/OpenClipartPlugin/spinnerLarge.gif"
                self.control.getControl("title" + str(i)).getModel().Label = ""
                i += 1
            if (self.page == "load"):
                searchQuery = self.control.getControl("search").getModel().Text
                data = Parser.load(str(searchQuery), 10, True)['payload']
                if (Parser.has_next_page()):
                    self.control.getControl("nextButton").getModel().Enabled = True
                else:
                    self.control.getControl("nextButton").getModel().Enabled = False
                self.control.getControl("previousButton").getModel().Enabled = False
            elif (self.page == "next"):
                data = Parser.next_page()['payload']
                if (Parser.has_next_page()):
                    self.control.getControl("nextButton").getModel().Enabled = True
                else:
                    self.control.getControl("nextButton").getModel().Enabled = False
                self.control.getControl("previousButton").getModel().Enabled = True
            elif (self.page == "previous"):
                data = Parser.previous_page()['payload']
                self.control.getControl("nextButton").getModel().Enabled = True
                if (Parser.has_previous_page()):
                    self.control.getControl("previousButton").getModel().Enabled = True
                else:
                    self.control.getControl("previousButton").getModel().Enabled = False
            if (not data is None):
                i = 1
                """
                We refresh the metadata matrix if it contains data.
                """
                if (MetadataContainer.hasData()):
                    MetadataContainer.refresh()
                while (i <= self.thumbs):
                    curImage = data[i-1]
                    self.control.getControl("title" + str(i)).getModel().Label = curImage['title']
                    png_url = curImage['svg']['png_thumb']
                    png_file = png_url[(png_url.rfind('/') + 1):]
                    thumb_url = "http://openclipart.org/image/100px/svg_to_png/" + png_file
                    self.control.getControl("image" + str(i)).getModel().ImageURL = thumb_url
                    i += 1
                    self.control.getControl("currentPage").getModel().Label = str(Parser.current_page())
                    self.control.getControl("totalPages").getModel().Label = str(Parser.total_pages())
                    """
                    We assign the meta data.
                    """
                    metadata = {
                        "url": curImage['svg']['url'],
                        "title": curImage['title'],
                        "uploader": curImage['uploader'],
                        "drawn_by": curImage['drawn_by'],
                        "description": curImage['description'],
                        "detail_link": curImage['detail_link'],
                        "created": curImage['created']
                    }
                    MetadataContainer.addData(metadata)
        except Exception as e:
            showMessage("Error", "ERROR: \n" + str(e))

"""
An event listener for the previous button.
"""
class MessageBoxListener( unohelper.Base, XActionListener ):
    def __init__(self, control_container):
        self.control = control_container

    def actionPerformed(self, actionEvent):
        self.control.dispose()

"""
Here we define a message box - mostly for debugging.
"""
def showMessage(msg, text):
    ctx = uno.getComponentContext()
    smgr = ctx.ServiceManager

    # create the dialog model and set the properties 
    dialogModel = smgr.createInstanceWithContext("com.sun.star.awt.UnoControlDialogModel", ctx)

    dialogModel.Width = 150
    dialogModel.Height = 75
    dialogModel.PositionX = 115
    dialogModel.PositionY = 15
    dialogModel.Title = msg

    # Label to hold the text
    labelModel = dialogModel.createInstance("com.sun.star.awt.UnoControlFixedTextModel" ); 
    labelModel.PositionX = 3
    labelModel.PositionY = 3
    labelModel.Width  = 94
    labelModel.Height = 94 
    labelModel.Name = "msgText"
    labelModel.TabIndex = 0
    labelModel.Label = text
    labelModel.MultiLine = True

    # create the button model model and set the properties 
    buttonModel = dialogModel.createInstance("com.sun.star.awt.UnoControlButtonModel" ); 
    buttonModel.PositionX = 25
    buttonModel.PositionY = 105
    buttonModel.Width  = 50
    buttonModel.Height = 15 
    buttonModel.TabIndex = 0
    buttonModel.Name = 'okButton'
    buttonModel.Label = 'OK'

    # insert the control models into the dialog model 
    dialogModel.insertByName("msgText", labelModel)
    dialogModel.insertByName("okButton", buttonModel)

    # create the dialog control and set the model 
    controlContainer = smgr.createInstanceWithContext("com.sun.star.awt.UnoControlDialog", ctx); 
    controlContainer.setModel(dialogModel); 

    controlContainer.getControl("okButton").addActionListener(MessageBoxListener(controlContainer))

    # create a peer 
    toolkit = smgr.createInstanceWithContext("com.sun.star.awt.ExtToolkit", ctx);

    controlContainer.setVisible(False);       
    controlContainer.createPeer(toolkit, None);

    # execute it
    controlContainer.execute()

    # dispose the dialog 
    controlContainer.dispose()

"""
Here we define the dialog to display the image thumbnails.
"""
def createDialog():
    imageModels = []
    labelModels = []
    ctx = uno.getComponentContext()
    smgr = ctx.ServiceManager

    # create the dialog model and set the properties 
    dialogModel = smgr.createInstanceWithContext("com.sun.star.awt.UnoControlDialogModel", ctx)

    dialogModel.Width = 115
    dialogModel.Height = 360
    dialogModel.PositionX = 105
    dialogModel.PositionY = 10
    dialogModel.Title = 'OpenClipart.org'

    # create the label model and set the properties 
    labelModel = dialogModel.createInstance("com.sun.star.awt.UnoControlFixedTextModel" ); 
    labelModel.PositionX = 3
    labelModel.PositionY = 6 
    labelModel.Width  = 33 
    labelModel.Height = 14 
    labelModel.Name = "SearchLabel" 
    labelModel.TabIndex = 1
    labelModel.Label = "Search tags: "

    # create the edit model and set the properties 
    editModel = dialogModel.createInstance( "com.sun.star.awt.UnoControlEditModel" ); 
    editModel.PositionX = 37
    editModel.PositionY = 6
    editModel.Width  = 55
    editModel.Height = 11 
    editModel.TabIndex = 2
    editModel.Name = 'search'
    editModel.Text = ''

    # create the button model model and set the properties 
    buttonModel = dialogModel.createInstance("com.sun.star.awt.UnoControlButtonModel" ); 
    buttonModel.PositionX = 92
    buttonModel.PositionY = 6
    buttonModel.Width  = 21
    buttonModel.Height = 11 
    buttonModel.TabIndex = 3
    buttonModel.Name = 'searchButton'
    buttonModel.Label = 'Search'

    # create the label model and set the properties 
    countModel = dialogModel.createInstance("com.sun.star.awt.UnoControlFixedTextModel" ); 
    countModel.PositionX = 3
    countModel.PositionY = 20
    countModel.Width  = 33 
    countModel.Height = 14 
    countModel.Name = "Counter" 
    countModel.Label = "Page:"

    # create the label model and set the properties 
    currentModel = dialogModel.createInstance("com.sun.star.awt.UnoControlFixedTextModel" ); 
    currentModel.PositionX = 22
    currentModel.PositionY = 20
    currentModel.Width  = 5
    currentModel.Height = 14
    currentModel.Align = 2
    currentModel.Name = "CurrentPage" 

    # create the label model and set the properties 
    ofModel = dialogModel.createInstance("com.sun.star.awt.UnoControlFixedTextModel" ); 
    ofModel.PositionX = 30
    ofModel.PositionY = 20
    ofModel.Width  = 5
    ofModel.Height = 14
    ofModel.Name = "Of" 
    ofModel.Label = "of" 

    # create the label model and set the properties 
    totalModel = dialogModel.createInstance("com.sun.star.awt.UnoControlFixedTextModel" ); 
    totalModel.PositionX = 37
    totalModel.PositionY = 20
    totalModel.Width  = 15
    totalModel.Height = 14
    totalModel.Name = "TotalPages" 

    # create the button model model and set the properties 
    previousModel = dialogModel.createInstance("com.sun.star.awt.UnoControlButtonModel" ); 
    previousModel.PositionX = 92
    previousModel.PositionY = 20
    previousModel.Width  = 11
    previousModel.Height = 11 
    previousModel.TabIndex = 4
    previousModel.Name = 'PreviousButton'
    previousModel.Label = '<-'
    previousModel.Enabled = False

    # create the button model model and set the properties 
    nextModel = dialogModel.createInstance("com.sun.star.awt.UnoControlButtonModel" ); 
    nextModel.PositionX = 103
    nextModel.PositionY = 20
    nextModel.Width  = 11
    nextModel.Height = 11 
    nextModel.TabIndex = 5
    nextModel.Name = 'NextButton'
    nextModel.Label = '->'
    nextModel.Enabled  = False

    # create the button model model and set the properties 
    lineModel = dialogModel.createInstance("com.sun.star.awt.UnoControlFixedLineModel" ); 
    lineModel.PositionX = 1
    lineModel.PositionY = 35
    lineModel.Width  = 111
    lineModel.Height = 2 
    lineModel.Name = 'Line'

    """
    In the following part we define the image models in which we will
    display the thumbnails.
    Furthermore we supply each image with a label model to declare the image
    title.
    """

    """
    First we declare a scroll bar to hold the images. If the images exeed
    the viewport the user can scroll up and down.
    This will come in usefull if we make it possible for the user to specify
    the number of images returned.
    """

    """
    We don't use this part at the moment as I can't seem to figure how to add elements
    scrollBar = dialogModel.createInstance("com.sun.star.awt.UnoControlScrollBarModel");
    scrollBar.PositionX = 0
    scrollBar.PositionY = 40
    scrollBar.Width  = 210
    scrollBar.Height = 665
    scrollBar.BlockIncrement = 103  # Fits one image
    scrollBar.Orientation = 1       # Vertical
    scrollBar.Name = 'scrollBar'
    """

    """
    Then we define the image models.
    """
    startPosX = 5
    startPosY = 43
    i = 1
    column = 1
    thumbs_to_show = 10         # For debugging we set this to 10
    while (i <= thumbs_to_show):
        curImage = dialogModel.createInstance("com.sun.star.awt.UnoControlButtonModel");
        curLabel = dialogModel.createInstance("com.sun.star.awt.UnoControlFixedTextModel");

        """
        We show two rows of images so we check if we have en odd or even
        number.
        """
        if (i % 2 == 1):        # Odd rows
            curImage.PositionX = curLabel.PositionX = startPosX
            curImage.PositionY = startPosY
            curLabel.PositionY = curImage.PositionY + 52
        else:
            curImage.PositionX = curLabel.PositionX = startPosX + 53
            curImage.PositionY = startPosY
            curLabel.PositionY = curImage.PositionY + 52
            column += 1
            startPosY += 63
        curImage.Width = 50
        curImage.Height = 50
        curImage.Name = 'image' + str(i)
        #scrollBar.insertByName('image' + str(i), curImage)
        dialogModel.insertByName('image' + str(i), curImage)
        curLabel.Width = 50
        curLabel.Height = 10
        curLabel.Name = 'title' + str(i)
        #scrollBar.insertByName('title' + str(i), curLabel)
        dialogModel.insertByName('title' + str(i), curLabel)
        i += 1

    # insert the control models into the dialog model 
    dialogModel.insertByName( "searchButton", buttonModel)
    dialogModel.insertByName( "searchLabel", labelModel)
    dialogModel.insertByName( "search", editModel)
    dialogModel.insertByName( "nextButton", nextModel)
    dialogModel.insertByName( "previousButton", previousModel)
    dialogModel.insertByName( "counter", countModel)
    dialogModel.insertByName( "currentPage", currentModel)
    dialogModel.insertByName( "of", ofModel)
    dialogModel.insertByName( "totalPages", totalModel)
    dialogModel.insertByName( "line", lineModel)
    #dialogModel.insertByName( "scrollBar", scrollBar)

    # create the dialog control and set the model 
    controlContainer = smgr.createInstanceWithContext("com.sun.star.awt.UnoControlDialog", ctx); 
    controlContainer.setModel(dialogModel); 

    # add the action listener
    """
    SearchModel = controlContainer.getControl("search").getModel()
    x = 1
    while (x <= thumbs_to_show):
        labelModels.append(controlContainer.getControl('title' + str(x)).getModel())
        imageModels.append(controlContainer.getControl('image' + str(x)).getModel())
        x += 1
    #SearchString = controlContainer.getControl("search").getModel().Text
    #SearchString = editModel.Text
    #SearchString = editModel.getText()
    """

    i = 1
    while (i <= thumbs_to_show):
        controlContainer.getControl('image'+str(i)).addActionListener(ThumbnailClickedListener(controlContainer, thumbs_to_show, i))
        i += 1

    controlContainer.getControl("searchButton").addActionListener(SearchClickedListener(controlContainer, thumbs_to_show))
    controlContainer.getControl("nextButton").addActionListener(ForwardClickedListener(controlContainer, thumbs_to_show))
    controlContainer.getControl("previousButton").addActionListener(BackwardClickedListener(controlContainer, thumbs_to_show))

    # create a peer 
    toolkit = smgr.createInstanceWithContext("com.sun.star.awt.ExtToolkit", ctx);

    controlContainer.setVisible(False);       
    controlContainer.createPeer(toolkit, None);

    # execute it
    controlContainer.execute()

    # dispose the dialog 
    controlContainer.dispose()

class Parser(object):
    #Class variables
    query_prefix = ""
    total_number_of_pages = 0
    total_number_of_api_pages = 0
    results_per_page = 0
    current_page_nr = 0
    next_page_nr = 0
    previous_page_nr = 0
    
    """
    A static method that initializes the load from openclipart.org.
    Furthermore control variable are set for the future readings.

    PARAMETERS
    query*              the search query from the user
    results_per_page    the number of results to be shown per page
                        this far we only handle values of tenths (10, 20 etc.)
    return_data         boolean that specifies whether to return the data or not
    
    * = required parameter
    
    RETURNS
    On success:         returns 1 or (if requested) the JSON data.
    On fail:            returns None. 
    """
    @classmethod
    def load(Parser, query, results_per_page=10, return_data=False):
        """
        We do the first initial load and set the appropriate variables.
        """

        """ We set the prefix for the query URL for later use. """
        Parser.query_prefix = 'http://openclipart.org/search/json/?query=' + query
        
        Parser.results_per_page = results_per_page
        Parser.current_page_nr = 1

        """ We parse the initial data load. """
        data = Parser.read(1)

        Parser.total_number_of_api_pages = data['info']['pages']
        Parser.total_number_of_pages = data['info']['pages'] / (Parser.results_per_page / 10)
        
        """ We set the initial pagination variables- """
        Parser.current_page_nr = 1
        Parser.previous_page_nr = 0
        Parser.next_page_nr = 1

        """
        When ocal receives a query with no matches it will throw a mysql
        warning, which of course is not valid JSON. Thus we have to check
        if our data object is None and if so we will return the same.
        """
        if (data is None):
            return None
        
        """
        Otherwise we proceed and check if the object contains any results. The
        mysql warning may be fixed at a moment and we have to look out for that.
        """
        if (data['info']['results'] == '0'):
            return None

        if (Parser.total_number_of_pages > 1):
            Parser.next_page_nr = 2

        if (return_data):
            return data
        else:
            return 1

    """
    A static method that reads data from a given page.

    PARAMETERS
    page_number*        the page number to show
    
    * = required parameter
    
    RETURNS
    The JSON data or None of no results were found.
    """
    @classmethod
    def read(Parser, page_number):
        """
        As for now the openclipart.org API does not have a reseults_per_page
        parameter and we thus have to accomolate the received data.
        """
        content_file = urlopen(Parser.query_prefix + "&page=" + str(page_number))
        json_string = content_file.read().decode('ISO-8859-1')

        if (json_string is None):
            return None

        if (Parser.results_per_page > 10):
            """
            In this loop we concatenate results for the requested number
            of pages.
            """
            tmp_data = json_string[:json_string.rfind('	]\
\
}')]
            i = 2
            while (i <= (Parser.results_per_page / 10)):
                if (page_number):
                    page = str(i)
                else:
                    page = str(i + (page_number * (Parser.results_per_page / 10)))
                cur_file = urlopen(Parser.query_prefix + "&page=" + page)
                cur_json = cur_file.read().decode('ISO-8859-1')
                tmp_data += cur_json[
                    cur_json.rfind('"payload" : [')
                    :
                    cur_json.rfind('	]\
\
}')
                ]
                i += 1
            #tmp_data += '	]\
#\
#}'

            """
            openclipart.org does not return entirely clean JSON, so we need to tell
            the json library to be less strict.
            """
            #TODO: this should be solved!
            data = json.loads(tmp_data, strict=False)
        else:
            data = json.loads(json_string, strict=False)
        Parser.current_page_nr = page_number
        if (Parser.current_page_nr > 0):
            Parser.previous_page_nr = Parser.current_page_nr - 1
        if (Parser.current_page_nr < Parser.total_number_of_pages):
            Parser.next_page_nr = Parser.current_page_nr + 1
        return data

    """
    A static method that returns the total number of pages.
    """
    @classmethod
    def total_pages(Parser):
        return int(Parser.total_number_of_pages)

    """
    A static method that returns the current page number.
    """
    @classmethod
    def current_page(Parser):
        return int(Parser.current_page_nr)

    """
    A static method that returns the data of the next page if any. Otherwise
    it returns None.
    """
    @classmethod
    def next_page(Parser):
        if (Parser.next_page_nr > 0):
            page_to_return = Parser.next_page_nr
            if (Parser.next_page_nr < Parser.total_number_of_pages):
                Parser.next_page_nr += 1
                Parser.current_page_nr += 1
                Parser.previous_page_nr += 1
            else:
                Parser.next_page_nr = 0
            return Parser.read(page_to_return)
        else:
            return None

    """
    A static method that returns the data of the previous page if any. Otherwise
    it returns None.
    """
    @classmethod
    def previous_page(Parser):
        if (Parser.previous_page_nr < Parser.total_number_of_pages):
            page_to_return = Parser.previous_page_nr
            if (Parser.previous_page_nr > 0):
                Parser.previous_page_nr -= 1
                Parser.current_page_nr -= 1
                Parser.next_page_nr -= 1
            else:
                Parser.previous_page_nr = 0
            return Parser.read(page_to_return)
        else:
            return None

    """
    A static method that returns if there are previous pages to show
    """
    @classmethod
    def has_previous_page(Parser):
        return Parser.previous_page_nr > 0

    """
    A static method that returns if there are more pages to show
    """
    @classmethod
    def has_next_page(Parser):
        return Parser.next_page_nr > 0

class MetadataContainer(object):
    metaMatrix = []

    """
    A static method that appends metadata to the .

    PARAMETERS
    dataTuple                   a tuple that represents the metadata of a give
                                image.
    """
    @classmethod
    def addData(MetadataContainer, dataTuple):
        MetadataContainer.metaMatrix.append(dataTuple)

    """
    A static method that returns the entire metadata matrix.

    RETURNS
    On success:         returns a copy of the metadata matrix.
    On fail:            returns None. 
    """
    @classmethod
    def getMetadataMatrix(MetadataContainer):
        if (len(MetadataContainer.metaMatrix) > 0):
            return MetadataContainer.metaMatrix
        else:
            return None

    """
    A static method that returns the metadata of a specific element.

    PARAMETERS
    index               the of the metadata to be read (0 based)
    
    RETURNS
    On success:         returns the metadata tuple.
    On fail:            returns None. 
    """
    @classmethod
    def getMetadata(MetadataContainer, index):
        return MetadataContainer.metaMatrix[index]

    """
    A static method used to refresh the metadata matrix.
    """
    @classmethod
    def refresh(MetadataContainer):
        MetadataContainer.metaMatrix = []

    """
    A static method used to check if the matrix contains elements.
    """
    @classmethod
    def hasData(MetadataContainer):
        return (len(MetadataContainer.metaMatrix) > 0)

g_exportedScripts = createDialog,
